import { DependencyManager } from "./dependency";
import * as async from "async";
export class ServiceContext {
    constructor() {
        this.services = {};
        this.dependencyManager = new DependencyManager;
    }
    registerService(s) {
        this.dependencyManager.add(s.name, s.dependencies || null);
        this.services[s.name] = s;
    }
    setup(finish = () => null) {
        let initialSequence = this.dependencyManager.resolve();
        let names = Object.keys(this.services);
        async.eachSeries(names, (name, done) => {
            let service = this.services[name];
            if (service.initialize) {
                if (service.initialize.length == 2) {
                    service.initialize(this.services, (err) => {
                        if (err) {
                            console.error(`Fail to initialize service ${name}.`);
                            done(err);
                            return;
                        }
                        console.log(`Service ${name} initialized`);
                        done();
                    });
                }
                else {
                    service.initialize(this.services);
                    console.log(`Service ${name} initialized`);
                    done();
                }
            }
            else {
                console.log(`Service ${name} initialized`);
                done();
            }
        }, (err) => {
            if (err) {
                finish(err);
                return;
            }
            console.log("All service initialized");
        });
    }
}
export class Service {
    constructor() {
        this.dependencies = [];
    }
}
